"""
This module defines a ``PermissionHandler`` abstraction that allows to
implement filter or access logic related to forums.
"""

from machina.apps.forum_permission.handler import (
    PermissionHandler as BasePermissionHandler,
)


class PermissionHandler(BasePermissionHandler):
    """Add ashley specific permission checks to the machina permission handler."""

    def can_rename_forum(self, forum, user):
        """ Given a forum, checks whether the user can rename it. """
        return self._perform_basic_permission_check(forum, user, "can_rename_forum")



"""
    Forum views
    ===========

    This module defines views provided by the ``forum`` application.

"""
from django.urls import reverse
from django.views.generic import UpdateView
from machina.apps.forum_permission.viewmixins import (
    PermissionRequiredMixin as BasePermissionRequiredMixin,
)
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model("forum", "Forum")

PermissionRequiredMixin: BasePermissionRequiredMixin = get_class(
    "forum_permission.viewmixins", "PermissionRequiredMixin"
)


class ForumRenameView(PermissionRequiredMixin, UpdateView):
    """ Allows the current user to update its forum profile. """

    model = Forum
    fields = ["name"]
    template_name = "forum/forum_rename.html"

    def get_success_url(self):
        """ Returns the success URL to redirect the user to. """
        return reverse(
            "forum:forum",
            kwargs={
                "pk": self.object.pk,
                "slug": self.object.slug,
            },
        )

    # pylint: disable=unused-argument
    def perform_permissions_check(self, user, obj, perms):
        """ Performs the permissions check. """
        return self.request.forum_permission_handler.can_rename_forum(obj, user)



"""
Ashley URLs (that includes django machina urls)
"""

from django.urls import include, path
from machina import urls as machina_urls

from ashley.views import ChangeUsernameView, ForumLTIView

urlpatterns = [
    path("lti/forum/<uuid:uuid>", ForumLTIView.as_view(), name="forum.lti.view"),
    path(
        "profile/username",
        ChangeUsernameView.as_view(),
        name="forum.username.change",
    ),
    path("forum/", include(machina_urls)),
]



"""
    Forum search forms
    ==================
    This module defines forms provided by the ``forum_search`` application.
"""

from machina.apps.forum_search.forms import SearchForm as MachinaSearchForm


class SearchForm(MachinaSearchForm):
    """
    Override Django Machina's SearchForm to allow searching poster user names
    even with an empty search in the main field.
    """

    def clean(self):
        """
        Set main query to catch all "*" if it is empty and there is a search term for
        poster username.
        """
        cleaned_data = super().clean()

        if cleaned_data.get("search_poster_name") and not cleaned_data.get("q"):
            cleaned_data["q"] = "*"

        return cleaned_data
